@@ -1,23 +1,27 @@
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import
+from __future__ import (absolute_import, division, print_function)
 
+from branca.colormap import (ColorMap, LinearColormap, StepColormap)
 from branca.element import (CssLink, Div, Element, Figure, Html, IFrame,
                             JavascriptLink, Link, MacroElement)
-from branca.colormap import (ColorMap, LinearColormap, StepColormap)
 
+from folium._version import get_versions
 
-from .folium import Map
-from .map import (
+from folium.features import (
+    Circle, CircleMarker, ClickForMarker, CustomIcon, DivIcon, GeoJson,
+    LatLngPopup, PolyLine, RegularPolygonMarker, TopoJson, Vega, VegaLite,
+    WmsTileLayer,
+)
+
+from folium.folium import Map
+
+from folium.map import (
     FeatureGroup, FitBounds, Icon, LayerControl, Marker, Popup, TileLayer
 )
-from .features import (
-    ClickForMarker, CustomIcon, DivIcon, GeoJson, LatLngPopup, CircleMarker,
-    MarkerCluster, PolyLine, Vega, RegularPolygonMarker,
-    TopoJson, WmsTileLayer
-)
 
-__version__ = '0.3.0'
+__version__ = get_versions()['version']
+del get_versions
 
 __all__ = [
     'CssLink',
@@ -33,6 +37,7 @@ __all__ = [
     'LinearColormap',
     'StepColormap',
     'Map',
+    'Circle',
     'CircleMarker',
     'RectangleMarker',
     'Polygon',
@@ -52,6 +57,7 @@ __all__ = [
     'MarkerCluster',
     'PolyLine',
     'Vega',
+    'VegaLite',
     'RegularPolygonMarker',
     'TopoJson',
     'WmsTileLayer'
