@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+
 """
 Features
 ------
@@ -7,20 +8,82 @@ Extra features Elements.
 
 """
 
+from __future__ import (absolute_import, division, print_function)
+
 import json
 
-from jinja2 import Template
-from six import text_type, binary_type
-
-from branca.utilities import (
-    _parse_size, _locations_mirror, _locations_tolist, image_to_url,
-    none_min, none_max, iter_points
-)
-from branca.element import (Element, Figure, JavascriptLink, CssLink,
-                            MacroElement)
 from branca.colormap import LinearColormap
+from branca.element import (CssLink, Element, Figure, JavascriptLink, MacroElement)  # noqa
+from branca.utilities import (_locations_tolist, _parse_size, image_to_url, iter_points, none_max, none_min)  # noqa
 
-from .map import Layer, Icon, Marker, Popup, FeatureGroup
+from folium.map import FeatureGroup, Icon, Layer, Popup, Marker
+from folium.utilities import _parse_path, _parse_wms
+
+from jinja2 import Template
+
+import requests
+
+from six import binary_type, text_type
+
+
+class PolyLine(Marker):
+    """
+    Creates a PolyLine (array) or MultiPolyline (array of arrays) object to
+    append into a map.
+
+    Parameters
+    ----------
+    locations: list of points (latitude, longitude)
+        Latitude and Longitude of line (Northing, Easting)
+    color: string, default Leaflet's default ('#03f')
+    weight: float, default Leaflet's default (5)
+    opacity: float, default Leaflet's default (0.5)
+    popup: string or folium.Popup, default None
+        Input text or visualization for object.
+
+    See http://leafletjs.com/reference-1.2.0.html#polyline for more options.
+
+    """
+    def __init__(self, locations, popup=None, **kw):
+        super(PolyLine, self).__init__(location=locations, popup=popup)
+        self._name = 'PolyLine'
+        options = _parse_path(**kw)
+        options.update(
+            {
+                'smoothFactor': kw.pop('smooth_factor', 1.0),
+                'noClip': kw.pop('no_clip', False),
+            }
+        )
+
+        self.options = json.dumps(options, sort_keys=True, indent=2)
+
+        self._template = Template(u"""
+            {% macro script(this, kwargs) %}
+                var {{this.get_name()}} = L.polyline(
+                    {{this.location}},
+                    {{ this.options }}).addTo({{this._parent.get_name()}});
+            {% endmacro %}
+            """)  # noqa
+
+    def _get_self_bounds(self):
+        """
+        Computes the bounds of the object itself (not including it's children)
+        in the form [[lat_min, lon_min], [lat_max, lon_max]]
+
+        """
+        bounds = [[None, None], [None, None]]
+        for point in iter_points(self.location):
+            bounds = [
+                [
+                    none_min(bounds[0][0], point[0]),
+                    none_min(bounds[0][1], point[1]),
+                ],
+                [
+                    none_max(bounds[1][0], point[0]),
+                    none_max(bounds[1][1], point[1]),
+                ],
+            ]
+        return bounds
 
 
 class WmsTileLayer(Layer):
@@ -33,13 +96,13 @@ class WmsTileLayer(Layer):
         The url of the WMS server.
     name : string, default None
         The name of the Layer, as it will appear in LayerControls
-    layers : str, default None
+    layers : str, default ''
         The names of the layers to be displayed.
-    styles : str, default None
+    styles : str, default ''
         Comma-separated list of WMS styles.
-    format : str, default None
+    fmt : str, default 'image/jpeg'
         The format of the service output.
-        Ex: 'iamge/png'
+        Ex: 'image/png'
     transparent: bool, default True
         Whether the layer shall allow transparency.
     version : str, default '1.1.1'
@@ -47,39 +110,33 @@ class WmsTileLayer(Layer):
     attr : str, default None
         The attribution of the service.
         Will be displayed in the bottom right corner.
-    overlay : bool, default False
+    overlay : bool, default True
         Adds the layer as an optional overlay (True) or the base layer (False).
     control : bool, default True
         Whether the Layer will be included in LayerControls
+    **kwargs : additional keyword arguments
+        Passed through to the underlying tileLayer.wms object and can be used
+        for setting extra tileLayer.wms parameters or as extra parameters in
+        the WMS request.
 
     For more information see:
     http://leafletjs.com/reference.html#tilelayer-wms
 
     """
-    def __init__(self, url, name=None, layers=None, styles=None, format=None,
-                 transparent=True, version='1.1.1', attr=None, overlay=True,
-                 control=True):
+    def __init__(self, url, name=None, attr='', overlay=True, control=True, **kwargs):  # noqa
         super(WmsTileLayer, self).__init__(overlay=overlay, control=control, name=name)  # noqa
         self.url = url
-        self.attribution = attr if attr is not None else ''
         # Options.
-        self.layers = layers if layers else ''
-        self.styles = styles if styles else ''
-        self.format = format if format else 'image/jpeg'
-        self.transparent = transparent
-        self.version = version
+        options = _parse_wms(**kwargs)
+        options.update({'attribution': attr})
+
+        self.options = json.dumps(options, sort_keys=True, indent=2)
+
         self._template = Template(u"""
         {% macro script(this, kwargs) %}
             var {{this.get_name()}} = L.tileLayer.wms(
                 '{{ this.url }}',
-                {
-                    layers: '{{ this.layers }}',
-                    styles: '{{ this.styles }}',
-                    format: '{{ this.format }}',
-                    transparent: {{ this.transparent.__str__().lower() }},
-                    version: '{{ this.version }}',
-                    {% if this.attribution %} attribution: '{{this.attribution}}'{% endif %}
-                    }
+                {{ this.options }}
                 ).addTo({{this._parent.get_name()}});
 
         {% endmacro %}
@@ -125,7 +182,10 @@ class RegularPolygonMarker(Marker):
     def __init__(self, location, color='black', opacity=1, weight=2,
                  fill_color='blue', fill_opacity=1,
                  number_of_sides=4, rotation=0, radius=15, popup=None):
-        super(RegularPolygonMarker, self).__init__(location, popup=popup)
+        super(RegularPolygonMarker, self).__init__(
+            _locations_tolist(location),
+            popup=popup
+        )
         self._name = 'RegularPolygonMarker'
         self.color = color
         self.opacity = opacity
@@ -161,11 +221,11 @@ class RegularPolygonMarker(Marker):
         super(RegularPolygonMarker, self).render()
 
         figure = self.get_root()
-        assert isinstance(figure, Figure), ("You cannot render this Element "
-                                            "if it's not in a Figure.")
+        assert isinstance(figure, Figure), ('You cannot render this Element '
+                                            'if it is not in a Figure.')
 
         figure.header.add_child(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet-dvf/0.3.0/leaflet-dvf.markers.min.js"),  # noqa
+            JavascriptLink('https://cdnjs.cloudflare.com/ajax/libs/leaflet-dvf/0.3.0/leaflet-dvf.markers.min.js'),  # noqa
             name='dvf_js')
 
 
@@ -199,11 +259,10 @@ class Vega(Element):
 
     """
     def __init__(self, data, width=None, height=None,
-                 left="0%", top="0%", position='relative'):
+                 left='0%', top='0%', position='relative'):
         super(Vega, self).__init__()
         self._name = 'Vega'
         self.data = data.to_json() if hasattr(data, 'to_json') else data
-        # FIXME:
         if isinstance(self.data, text_type) or isinstance(data, binary_type):
             self.data = json.loads(self.data)
 
@@ -215,7 +274,7 @@ class Vega(Element):
         self.left = _parse_size(left)
         self.top = _parse_size(top)
         self.position = position
-        self._template = Template(u"")
+        self._template = Template(u'')
 
     def render(self, **kwargs):
         """Renders the HTML representation of the element."""
@@ -230,8 +289,8 @@ class Vega(Element):
             """).render(this=self)), name=self.get_name())
 
         figure = self.get_root()
-        assert isinstance(figure, Figure), ("You cannot render this Element "
-                                            "if it's not in a Figure.")
+        assert isinstance(figure, Figure), ('You cannot render this Element '
+                                            'if it is not in a Figure.')
 
         figure.header.add_child(Element(Template("""
             <style> #{{this.get_name()}} {
@@ -244,15 +303,15 @@ class Vega(Element):
             """).render(this=self, **kwargs)), name=self.get_name())
 
         figure.header.add_child(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/d3/3.5.5/d3.min.js"),  # noqa
+            JavascriptLink('https://cdnjs.cloudflare.com/ajax/libs/d3/3.5.5/d3.min.js'),  # noqa
             name='d3')
 
         figure.header.add_child(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/vega/1.4.3/vega.min.js"),  # noqa
+            JavascriptLink('https://cdnjs.cloudflare.com/ajax/libs/vega/1.4.3/vega.min.js'),  # noqa
             name='vega')
 
         figure.header.add_child(
-            JavascriptLink("https://code.jquery.com/jquery-2.1.0.min.js"),
+            JavascriptLink('https://code.jquery.com/jquery-2.1.0.min.js'),
             name='jquery')
 
         figure.script.add_child(
@@ -261,6 +320,102 @@ class Vega(Element):
             name='vega_parse')
 
 
+class VegaLite(Element):
+    """
+    Creates a Vega-Lite chart element.
+
+    Parameters
+    ----------
+    data: JSON-like str or object
+        The Vega-Lite description of the chart.
+        It can also be any object that has a method `to_json`,
+        so that you can (for instance) provide an `Altair` chart.
+    width: int or str, default None
+        The width of the output element.
+        If None, either data['width'] (if available) or '100%' will be used.
+        Ex: 120, '120px', '80%'
+    height: int or str, default None
+        The height of the output element.
+        If None, either data['width'] (if available) or '100%' will be used.
+        Ex: 120, '120px', '80%'
+    left: int or str, default '0%'
+        The horizontal distance of the output with respect to the parent
+        HTML object. Ex: 120, '120px', '80%'
+    top: int or str, default '0%'
+        The vertical distance of the output with respect to the parent
+        HTML object. Ex: 120, '120px', '80%'
+    position: str, default 'relative'
+        The `position` argument that the CSS shall contain.
+        Ex: 'relative', 'absolute'
+
+    """
+    def __init__(self, data, width=None, height=None,
+                 left='0%', top='0%', position='relative'):
+        super(VegaLite, self).__init__()
+        self._name = 'VegaLite'
+        self.data = data.to_json() if hasattr(data, 'to_json') else data
+        if isinstance(self.data, text_type) or isinstance(data, binary_type):
+            self.data = json.loads(self.data)
+
+        # Size Parameters.
+        self.width = _parse_size(self.data.get('width', '100%') if
+                                 width is None else width)
+        self.height = _parse_size(self.data.get('height', '100%') if
+                                  height is None else height)
+        self.left = _parse_size(left)
+        self.top = _parse_size(top)
+        self.position = position
+        self._template = Template(u'')
+
+    def render(self, **kwargs):
+        """Renders the HTML representation of the element."""
+        self.json = json.dumps(self.data)
+
+        self._parent.html.add_child(Element(Template("""
+            <div id="{{this.get_name()}}"></div>
+            """).render(this=self, kwargs=kwargs)), name=self.get_name())
+
+        self._parent.script.add_child(Element(Template("""
+            var embedSpec = {
+                mode: "vega-lite",
+                spec: {{this.json}}
+            };
+            vg.embed(
+                {{this.get_name()}}, embedSpec, function(error, result) {}
+            );
+        """).render(this=self)), name=self.get_name())
+
+        figure = self.get_root()
+        assert isinstance(figure, Figure), ('You cannot render this Element '
+                                            'if it is not in a Figure.')
+
+        figure.header.add_child(Element(Template("""
+            <style> #{{this.get_name()}} {
+                position : {{this.position}};
+                width : {{this.width[0]}}{{this.width[1]}};
+                height: {{this.height[0]}}{{this.height[1]}};
+                left: {{this.left[0]}}{{this.left[1]}};
+                top: {{this.top[0]}}{{this.top[1]}};
+            </style>
+            """).render(this=self, **kwargs)), name=self.get_name())
+
+        figure.header.add_child(
+            JavascriptLink('https://d3js.org/d3.v3.min.js'),
+            name='d3')
+
+        figure.header.add_child(
+            JavascriptLink('https://cdnjs.cloudflare.com/ajax/libs/vega/2.6.5/vega.min.js'),  # noqa
+            name='vega')
+
+        figure.header.add_child(
+            JavascriptLink('https://cdnjs.cloudflare.com/ajax/libs/vega-lite/1.3.1/vega-lite.min.js'),  # noqa
+            name='vega-lite')
+
+        figure.header.add_child(
+            JavascriptLink('https://cdnjs.cloudflare.com/ajax/libs/vega-embed/2.2.0/vega-embed.min.js'),  # noqa
+            name='vega-embed')
+
+
 class GeoJson(Layer):
     """
     Creates a GeoJson object for plotting into a Map.
@@ -275,7 +430,9 @@ class GeoJson(Layer):
         in the JavaScript.
         * If str, then data will be passed to the JavaScript as-is.
     style_function: function, default None
-        A function mapping a GeoJson Feature to a style dict.
+        Function mapping a GeoJson Feature to a style dict.
+    highlight_function: function, default None
+        Function mapping a GeoJson Feature to a style dict for mouse events.
     name : string, default None
         The name of the Layer, as it will appear in LayerControls
     overlay : bool, default False
@@ -311,19 +468,18 @@ class GeoJson(Layer):
         super(GeoJson, self).__init__(name=name, overlay=overlay,
                                       control=control)
         self._name = 'GeoJson'
-        if hasattr(data, 'read'):
-            self.embed = True
-            self.data = json.load(data)
-        elif isinstance(data, dict):
+        if isinstance(data, dict):
             self.embed = True
             self.data = data
         elif isinstance(data, text_type) or isinstance(data, binary_type):
-            if data.lstrip()[0] in '[{':  # This is a GeoJSON inline string
-                self.embed = True
+            self.embed = True
+            if data.lower().startswith(('http:', 'ftp:', 'https:')):
+                self.data = requests.get(data).json()
+            elif data.lstrip()[0] in '[{':  # This is a GeoJSON inline string
                 self.data = json.loads(data)
             else:  # This is a filename
-                self.embed = False
-                self.data = data
+                with open(data) as f:
+                    self.data = json.loads(f.read())
         elif data.__class__.__name__ in ['GeoDataFrame', 'GeoSeries']:
             self.embed = True
             if hasattr(data, '__geo_interface__'):
@@ -548,11 +704,11 @@ class TopoJson(Layer):
         super(TopoJson, self).render(**kwargs)
 
         figure = self.get_root()
-        assert isinstance(figure, Figure), ("You cannot render this Element "
-                                            "if it's not in a Figure.")
+        assert isinstance(figure, Figure), ('You cannot render this Element '
+                                            'if it is not in a Figure.')
 
         figure.header.add_child(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/topojson/1.6.9/topojson.min.js"),  # noqa
+            JavascriptLink('https://cdnjs.cloudflare.com/ajax/libs/topojson/1.6.9/topojson.min.js'),  # noqa
             name='topojson')
 
     def _get_self_bounds(self):
@@ -588,56 +744,6 @@ class TopoJson(Layer):
         ]
 
 
-class MarkerCluster(Layer):
-    """
-    Creates a MarkerCluster element to append into a map with
-    Map.add_child.
-
-    Parameters
-    ----------
-    name : string, default None
-        The name of the Layer, as it will appear in LayerControls
-    overlay : bool, default False
-        Adds the layer as an optional overlay (True) or the base layer (False).
-    control : bool, default True
-        Whether the Layer will be included in LayerControls
-
-    """
-    def __init__(self, name=None, overlay=True, control=True):
-        super(MarkerCluster, self).__init__(name=name, overlay=overlay,
-                                            control=control)
-        self._name = 'MarkerCluster'
-        self._template = Template(u"""
-            {% macro script(this, kwargs) %}
-            var {{this.get_name()}} = L.markerClusterGroup();
-            {{this._parent.get_name()}}.addLayer({{this.get_name()}});
-            {% endmacro %}
-            """)
-
-    def render(self, **kwargs):
-        """Renders the HTML representation of the element."""
-        super(MarkerCluster, self).render()
-
-        figure = self.get_root()
-        assert isinstance(figure, Figure), ("You cannot render this Element "
-                                            "if it's not in a Figure.")
-        figure.header.add_child(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/leaflet.markercluster-src.js"),  # noqa
-            name='marker_cluster_src')
-
-        figure.header.add_child(
-            JavascriptLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/leaflet.markercluster.js"),  # noqa
-            name='marker_cluster')
-
-        figure.header.add_child(
-            CssLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/MarkerCluster.css"),  # noqa
-            name='marker_cluster_css')
-
-        figure.header.add_child(
-            CssLink("https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/MarkerCluster.Default.css"),  # noqa
-            name="marker_cluster_default_css")
-
-
 class DivIcon(MacroElement):
     """
     Represents a lightweight icon for markers that uses a simple `div`
@@ -697,43 +803,40 @@ class Circle(Marker):
 
     Parameters
     ----------
-    location: tuple or list, default None
+    location: tuple or list
         Latitude and Longitude of Marker (Northing, Easting)
     radius: int
-        The radius of the circle in meters. For setting the radius in pixel,
-        use CircleMarker.
-    color: str, default 'black'
+        The radius of the circle in meters.
+        For setting the radius in pixel, use CircleMarker.
+    color: str, default '#3388ff'
         The color of the marker's edge in a HTML-compatible format.
-    fill_color: str, default 'black'
+    fill: bool, default False
+        If true the circle will be filled.
+    fill_color: str, default to the same as color
         The fill color of the marker in a HTML-compatible format.
-    fill_opacity: float, default 0.6
+    fill_opacity: float, default 0.2
         The fill opacity of the marker, between 0. and 1.
     popup: string or folium.Popup, default None
         Input text or visualization for object.
 
+    See http://leafletjs.com/reference-1.2.0.html#path for more otions.
+
     """
-    def __init__(self, location, radius=500, color='black',
-                 fill_color='black', fill_opacity=0.6, popup=None):
-        super(Circle, self).__init__(location, popup=popup)
-        self._name = 'Circle'
-        self.radius = radius
-        self.color = color
-        self.fill_color = fill_color
-        self.fill_opacity = fill_opacity
+    def __init__(self, location, radius=10, popup=None, **kwargs):
+        super(Circle, self).__init__(location=location, popup=popup)
+        self._name = 'circle'
+        options = _parse_path(**kwargs)
+
+        options.update({'radius': radius})
+        self.options = json.dumps(options, sort_keys=True, indent=2)
 
         self._template = Template(u"""
             {% macro script(this, kwargs) %}
 
             var {{this.get_name()}} = L.circle(
                 [{{this.location[0]}},{{this.location[1]}}],
-                {{ this.radius }},
-                {
-                    color: '{{ this.color }}',
-                    fillColor: '{{ this.fill_color }}',
-                    fillOpacity: {{ this.fill_opacity }}
-                    }
-                )
-                .addTo({{this._parent.get_name()}});
+                {{ this.options }}
+                ).addTo({{this._parent.get_name()}});
             {% endmacro %}
             """)
 
@@ -744,48 +847,39 @@ class CircleMarker(Marker):
 
     Parameters
     ----------
-    location: tuple or list, default None
+    location: tuple or list
         Latitude and Longitude of Marker (Northing, Easting)
     radius: int
-        The radius of the circle in pixels. For setting the radius in meter,
-        use Circle.
-    color: str, default 'black'
+        The radius of the circle in pixels.
+        For setting the radius in meter, use Circle.
+    color: str, default '#3388ff'
         The color of the marker's edge in a HTML-compatible format.
-    weight: int, default 2
-        Stroke weight in pixels
-    fill_color: str, default 'black'
+    fill: bool, default False
+        If true the circle will be filled.
+    fill_color: str, default to the same as color
         The fill color of the marker in a HTML-compatible format.
-    fill_opacity: float, default 0.6
+    fill_opacity: float, default 0.2
         The fill opacity of the marker, between 0. and 1.
     popup: string or folium.Popup, default None
         Input text or visualization for object.
 
+    See http://leafletjs.com/reference-1.2.0.html#path for more otions.
+
     """
-    def __init__(self, location, radius=500, color='black',
-                 weight=2, fill_color='black', fill_opacity=0.6,
-                 popup=None):
-        super(CircleMarker, self).__init__(location, popup=popup)
+    def __init__(self, location, radius=10, popup=None, **kw):
+        super(CircleMarker, self).__init__(location=location, popup=popup)
         self._name = 'CircleMarker'
-        self.radius = radius
-        self.color = color
-        self.weight = weight
-        self.fill_color = fill_color
-        self.fill_opacity = fill_opacity
+        options = _parse_path(**kw)
+
+        options.update({'radius': radius})
+        self.options = json.dumps(options, sort_keys=True, indent=2)
 
         self._template = Template(u"""
             {% macro script(this, kwargs) %}
-
             var {{this.get_name()}} = L.circleMarker(
                 [{{this.location[0]}},{{this.location[1]}}],
-                {
-                    color: '{{ this.color }}',
-                    weight: {{ this.weight }},
-                    fillColor: '{{ this.fill_color }}',
-                    fillOpacity: {{ this.fill_opacity }}
-                    }
-                )
-                .setRadius({{ this.radius }})
-                .addTo({{this._parent.get_name()}});
+                {{ this.options }}
+                ).addTo({{this._parent.get_name()}});
             {% endmacro %}
             """)
 
@@ -847,7 +941,7 @@ class RectangleMarker(Marker):
 
 class PolygonMarker(Marker):
     def __init__(self, locations, color='black', weight=1, fill_color='black',
-                 fill_opacity=0.6, popup=None, latlon=True):
+                 fill_opacity=0.6, popup=None):
         """
         Creates a PolygonMarker object for plotting on a Map.
 
@@ -881,10 +975,7 @@ class PolygonMarker(Marker):
         ...         fill_opacity=0.5, popup='Tokyo, Japan'))
 
         """
-        super(PolygonMarker, self).__init__((
-            _locations_mirror(locations) if not latlon else
-            _locations_tolist(locations)), popup=popup
-        )
+        super(PolygonMarker, self).__init__(locations, popup=popup)
         self._name = 'PolygonMarker'
         self.color = color
         self.weight = weight
@@ -964,75 +1055,6 @@ class ClickForMarker(MacroElement):
             """)  # noqa
 
 
-class PolyLine(MacroElement):
-    """
-    Creates a PolyLine object to append into a map with
-    Map.add_child.
-
-    Parameters
-    ----------
-    locations: list of points (latitude, longitude)
-        Latitude and Longitude of line (Northing, Easting)
-    color: string, default Leaflet's default ('#03f')
-    weight: float, default Leaflet's default (5)
-    opacity: float, default Leaflet's default (0.5)
-    latlon: bool, default True
-        Whether locations are given in the form [[lat, lon]]
-        or not ([[lon, lat]] if False).
-        Note that the default GeoJson format is latlon=False,
-        while Leaflet polyline's default is latlon=True.
-    popup: string or folium.Popup, default None
-        Input text or visualization for object.
-
-    """
-    def __init__(self, locations, color=None, weight=None,
-                 opacity=None, latlon=True, popup=None):
-        super(PolyLine, self).__init__()
-        self._name = 'PolyLine'
-        self.data = (_locations_mirror(locations) if not latlon else
-                     _locations_tolist(locations))
-        self.color = color
-        self.weight = weight
-        self.opacity = opacity
-        if isinstance(popup, text_type) or isinstance(popup, binary_type):
-            self.add_child(Popup(popup))
-        elif popup is not None:
-            self.add_child(popup)
-
-        self._template = Template(u"""
-            {% macro script(this, kwargs) %}
-                var {{this.get_name()}} = L.polyline(
-                    {{this.data}},
-                    {
-                        {% if this.color != None %}color: '{{ this.color }}',{% endif %}
-                        {% if this.weight != None %}weight: {{ this.weight }},{% endif %}
-                        {% if this.opacity != None %}opacity: {{ this.opacity }},{% endif %}
-                        });
-                {{this._parent.get_name()}}.addLayer({{this.get_name()}});
-            {% endmacro %}
-            """)  # noqa
-
-    def _get_self_bounds(self):
-        """
-        Computes the bounds of the object itself (not including it's children)
-        in the form [[lat_min, lon_min], [lat_max, lon_max]]
-
-        """
-        bounds = [[None, None], [None, None]]
-        for point in iter_points(self.data):
-            bounds = [
-                [
-                    none_min(bounds[0][0], point[0]),
-                    none_min(bounds[0][1], point[1]),
-                ],
-                [
-                    none_max(bounds[1][0], point[0]),
-                    none_max(bounds[1][1], point[1]),
-                ],
-            ]
-        return bounds
-
-
 class CustomIcon(Icon):
     """
     Create a custom icon, based on an image.
