@@ -8,38 +8,40 @@ Classes for drawing maps.
 
 """
 
-from __future__ import unicode_literals
+from __future__ import (absolute_import, division, print_function)
 
 import json
+import os
+import tempfile
+import time
+
 from collections import OrderedDict
 
+from branca.element import CssLink, Element, Figure, Html, JavascriptLink, MacroElement  # noqa
+from branca.utilities import _parse_size
+
+from folium.utilities import _validate_coordinates, _validate_location
+
 from jinja2 import Environment, PackageLoader, Template
 
-from branca.six import text_type, binary_type
-from branca.utilities import _parse_size
-from branca.element import (Element, Figure, MacroElement, Html,
-                            JavascriptLink, CssLink)
+from six import binary_type, text_type
 
 ENV = Environment(loader=PackageLoader('folium', 'templates'))
 
 _default_js = [
     ('leaflet',
-     'https://unpkg.com/leaflet@1.0.1/dist/leaflet.js'),
+     'https://cdn.jsdelivr.net/npm/leaflet@1.2.0/dist/leaflet.js'),
     ('jquery',
      'https://ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js'),
     ('bootstrap',
      'https://maxcdn.bootstrapcdn.com/bootstrap/3.2.0/js/bootstrap.min.js'),
     ('awesome_markers',
      'https://cdnjs.cloudflare.com/ajax/libs/Leaflet.awesome-markers/2.0.2/leaflet.awesome-markers.js'),  # noqa
-    ('marker_cluster_src',
-     'https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/leaflet.markercluster-src.js'),  # noqa
-    ('marker_cluster',
-     'https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/leaflet.markercluster.js'),  # noqa
     ]
 
 _default_css = [
     ('leaflet_css',
-     'https://unpkg.com/leaflet@1.0.1/dist/leaflet.css'),
+     'https://cdn.jsdelivr.net/npm/leaflet@1.2.0/dist/leaflet.css'),
     ('bootstrap_css',
      'https://maxcdn.bootstrapcdn.com/bootstrap/3.2.0/css/bootstrap.min.css'),
     ('bootstrap_theme_css',
@@ -48,10 +50,6 @@ _default_css = [
      'https://maxcdn.bootstrapcdn.com/font-awesome/4.6.3/css/font-awesome.min.css'),  # noqa
     ('awesome_markers_css',
      'https://cdnjs.cloudflare.com/ajax/libs/Leaflet.awesome-markers/2.0.2/leaflet.awesome-markers.css'),  # noqa
-    ('marker_cluster_default_css',
-     'https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/MarkerCluster.Default.css'),  # noqa
-    ('marker_cluster_css',
-     'https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.0.0/MarkerCluster.css'),  # noqa
     ('awesome_rotate_css',
      'https://rawgit.com/python-visualization/folium/master/folium/templates/leaflet.awesome.rotate.css'),  # noqa
     ]
@@ -146,23 +144,27 @@ class LegacyMap(MacroElement):
     ...                        attr='Mapbox attribution')
     """
     def __init__(self, location=None, width='100%', height='100%',
-                 left="0%", top="0%", position='relative',
+                 left='0%', top='0%', position='relative',
                  tiles='OpenStreetMap', API_key=None, max_zoom=18, min_zoom=1,
-                 zoom_start=10, continuous_world=False, world_copy_jump=False,
+                 zoom_start=10, world_copy_jump=False,
                  no_wrap=False, attr=None, min_lat=-90, max_lat=90,
-                 min_lon=-180, max_lon=180, max_bounds=True,
+                 min_lon=-180, max_lon=180, max_bounds=False,
                  detect_retina=False, crs='EPSG3857', control_scale=False,
-                 prefer_canvas=False, no_touch=False, disable_3d=False):
+                 prefer_canvas=False, no_touch=False, disable_3d=False,
+                 subdomains='abc', png_enabled=False):
         super(LegacyMap, self).__init__()
         self._name = 'Map'
         self._env = ENV
+        # Undocumented for now b/c this will be subject to a re-factor soon.
+        self._png_image = None
+        self.png_enabled = png_enabled
 
         if not location:
             # If location is not passed we center and ignore zoom.
             self.location = [0, 0]
             self.zoom_start = min_zoom
         else:
-            self.location = location
+            self.location = _validate_location(location)
             self.zoom_start = zoom_start
 
         Figure().add_child(self)
@@ -179,20 +181,24 @@ class LegacyMap(MacroElement):
         self.min_lon = min_lon
         self.max_lon = max_lon
         self.max_bounds = max_bounds
-        self.continuous_world = continuous_world
         self.no_wrap = no_wrap
         self.world_copy_jump = world_copy_jump
 
         self.crs = crs
         self.control_scale = control_scale
 
-        self.global_switches = GlobalSwitches(prefer_canvas, no_touch, disable_3d)
+        self.global_switches = GlobalSwitches(
+            prefer_canvas,
+            no_touch,
+            disable_3d
+        )
 
         if tiles:
             self.add_tile_layer(
                 tiles=tiles, min_zoom=min_zoom, max_zoom=max_zoom,
-                continuous_world=continuous_world, no_wrap=no_wrap, attr=attr,
-                API_key=API_key, detect_retina=detect_retina
+                no_wrap=no_wrap, attr=attr,
+                API_key=API_key, detect_retina=detect_retina,
+                subdomains=subdomains
             )
 
         self._template = Template(u"""
@@ -234,8 +240,7 @@ class LegacyMap(MacroElement):
         """)  # noqa
 
     def _repr_html_(self, **kwargs):
-        """Displays the Map in a Jupyter notebook.
-        """
+        """Displays the HTML Map in a Jupyter notebook."""
         if self._parent is None:
             self.add_to(Figure())
             out = self._parent._repr_html_(**kwargs)
@@ -244,10 +249,42 @@ class LegacyMap(MacroElement):
             out = self._parent._repr_html_(**kwargs)
         return out
 
+    def _to_png(self):
+        """Export the HTML to byte representation of a PNG image."""
+        if self._png_image is None:
+            import selenium.webdriver
+
+            with tempfile.NamedTemporaryFile(suffix='.html') as f:
+                fname = f.name
+                self.save(fname, close_file=False)
+                driver = selenium.webdriver.PhantomJS(
+                    service_log_path=os.path.devnull
+                )
+                driver.get('file://{}'.format(fname))
+                driver.maximize_window()
+                # Ignore user map size.
+                driver.execute_script("document.body.style.width = '100%';")  # noqa
+                # We should probably monitor if some element is present,
+                # but this is OK for now.
+                time.sleep(3)
+                png = driver.get_screenshot_as_png()
+                driver.quit()
+                self._png_image = png
+        return self._png_image
+
+    def _repr_png_(self):
+        """Displays the PNG Map in a Jupyter notebook."""
+        # The notebook calls all _repr_*_ by default.
+        # We don't want that here b/c this one is quite slow.
+        if not self.png_enabled:
+            return None
+        return self._to_png()
+
     def add_tile_layer(self, tiles='OpenStreetMap', name=None,
                        API_key=None, max_zoom=18, min_zoom=1,
-                       continuous_world=False, attr=None, active=False,
-                       detect_retina=False, no_wrap=False, **kwargs):
+                       attr=None, active=False,
+                       detect_retina=False, no_wrap=False, subdomains='abc',
+                       **kwargs):
         """
         Add a tile layer to the map. See TileLayer for options.
 
@@ -256,15 +293,15 @@ class LegacyMap(MacroElement):
                                min_zoom=min_zoom, max_zoom=max_zoom,
                                attr=attr, API_key=API_key,
                                detect_retina=detect_retina,
-                               continuous_world=continuous_world,
+                               subdomains=subdomains,
                                no_wrap=no_wrap)
         self.add_child(tile_layer, name=tile_layer.tile_name)
 
     def render(self, **kwargs):
         """Renders the HTML representation of the element."""
         figure = self.get_root()
-        assert isinstance(figure, Figure), ("You cannot render this Element "
-                                            "if it's not in a Figure.")
+        assert isinstance(figure, Figure), ('You cannot render this Element '
+                                            'if it is not in a Figure.')
 
         # Set global switches
         figure.header.add_child(self.global_switches, name='global_switches')
@@ -374,11 +411,13 @@ class TileLayer(Layer):
         Adds the layer as an optional overlay (True) or the base layer (False).
     control : bool, default True
         Whether the Layer will be included in LayerControls.
+    subdomains: list of strings, default ['abc']
+        Subdomains of the tile service.
     """
     def __init__(self, tiles='OpenStreetMap', min_zoom=1, max_zoom=18,
                  attr=None, API_key=None, detect_retina=False,
-                 continuous_world=False, name=None, overlay=False,
-                 control=True, no_wrap=False):
+                 name=None, overlay=False,
+                 control=True, no_wrap=False, subdomains='abc'):
         self.tile_name = (name if name is not None else
                           ''.join(tiles.lower().strip().split()))
         super(TileLayer, self).__init__(name=self.tile_name, overlay=overlay,
@@ -386,12 +425,15 @@ class TileLayer(Layer):
         self._name = 'TileLayer'
         self._env = ENV
 
-        self.min_zoom = min_zoom
-        self.max_zoom = max_zoom
-        self.no_wrap = no_wrap
-        self.continuous_world = continuous_world
-
-        self.detect_retina = detect_retina
+        options = {
+            'minZoom': min_zoom,
+            'maxZoom': max_zoom,
+            'noWrap': no_wrap,
+            'attribution': attr,
+            'subdomains': subdomains,
+            'detectRetina': detect_retina,
+        }
+        self.options = json.dumps(options, sort_keys=True, indent=2)
 
         self.tiles = ''.join(tiles.lower().strip().split())
         if self.tiles in ('cloudmade', 'mapbox') and not API_key:
@@ -418,16 +460,8 @@ class TileLayer(Layer):
         {% macro script(this, kwargs) %}
             var {{this.get_name()}} = L.tileLayer(
                 '{{this.tiles}}',
-                {
-                    maxZoom: {{this.max_zoom}},
-                    minZoom: {{this.min_zoom}},
-                    continuousWorld: {{this.continuous_world.__str__().lower()}},
-                    noWrap: {{this.no_wrap.__str__().lower()}},
-                    attribution: '{{this.attr}}',
-                    detectRetina: {{this.detect_retina.__str__().lower()}}
-                    }
+                {{ this.options }}
                 ).addTo({{this._parent.get_name()}});
-
         {% endmacro %}
         """)  # noqa
 
@@ -581,7 +615,8 @@ class Icon(MacroElement):
 
 
 class Marker(MacroElement):
-    """Create a simple stock Leaflet marker on the map, with optional
+    """
+    Create a simple stock Leaflet marker on the map, with optional
     popup text or Vincent visualization.
 
     Parameters
@@ -601,11 +636,14 @@ class Marker(MacroElement):
     --------
     >>> Marker(location=[45.5, -122.3], popup='Portland, OR')
     >>> Marker(location=[45.5, -122.3], popup=folium.Popup('Portland, OR'))
+
     """
     def __init__(self, location, popup=None, icon=None):
         super(Marker, self).__init__()
         self._name = 'Marker'
-        self.location = location
+        # Must be _validate_coordinates b/c some markers are defined with
+        # multiple coordinates values, like Polygons.
+        self.location = _validate_coordinates(location)
         if icon is not None:
             self.add_child(icon)
         if isinstance(popup, text_type) or isinstance(popup, binary_type):
@@ -641,10 +679,12 @@ class Popup(Element):
     ----------
     html: string or Element
         Content of the Popup.
+    parse_html: bool, default False
+        True if the popup is a template that needs to the rendered first.
     max_width: int, default 300
         The maximal width of the popup.
     """
-    def __init__(self, html=None, max_width=300):
+    def __init__(self, html=None, parse_html=False, max_width=300):
         super(Popup, self).__init__()
         self._name = 'Popup'
         self.header = Element()
@@ -655,10 +695,12 @@ class Popup(Element):
         self.html._parent = self
         self.script._parent = self
 
+        script = not parse_html
+
         if isinstance(html, Element):
             self.html.add_child(html)
         elif isinstance(html, text_type) or isinstance(html, binary_type):
-            self.html.add_child(Html(text_type(html)))
+            self.html.add_child(Html(text_type(html), script=script))
 
         self.max_width = max_width
 
@@ -683,8 +725,8 @@ class Popup(Element):
             child.render(**kwargs)
 
         figure = self.get_root()
-        assert isinstance(figure, Figure), ("You cannot render this Element "
-                                            "if it's not in a Figure.")
+        assert isinstance(figure, Figure), ('You cannot render this Element '
+                                            'if it is not in a Figure.')
 
         figure.script.add_child(Element(
             self._template.render(this=self, kwargs=kwargs)),
