@@ -1,19 +1,15 @@
 # -*- coding: utf-8 -*-
-"""
-ScrollZoomToggler plugin
-------------------------
 
-Adds a button to enable/disable zoom scrolling.
-"""
-from jinja2 import Template
+from __future__ import (absolute_import, division, print_function)
 
 from branca.element import MacroElement
 
+from jinja2 import Template
+
 
 class ScrollZoomToggler(MacroElement):
+    """Creates a button for enabling/disabling scroll on the Map."""
     def __init__(self):
-        """Creates a button for enabling/disabling scroll on the Map.
-        """
         super(ScrollZoomToggler, self).__init__()
         self._name = 'ScrollZoomToggler'
 
